#include <linux/config.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/socket.h>
#include <linux/in.h>

#include <pthread.h>
#include <time.h>
#include <unistd.h>

#include <asm/system.h>
#include <asm/io.h>

#include <rtl.h>
#include <rtl_fifo.h>

#define RTL_SPIN_LOCK SPIN_LOCK_UNLOCKED
#include <rtl_sync.h>
#include "../ocs.h"


MODULE_AUTHOR("Pierre Morel <pmorel@mnis.fr>");
MODULE_DESCRIPTION("Ocera Network ping module example");
MODULE_LICENSE("GPL");

/*
 * Initialize the module, get queues and wait for data
 */

int period = OCS_PERIOD*100 ;

pthread_t	ping_thread;
char * mon_message = "salut";
extern ocs_t	* my_stream_p;



void *ping_rt_thread(void *t){
	ocera_msgbuf_t *mbp;

	ODEBUGP(ODEB_CALL,( KERN_INFO "PING: start %p\n",t));

	ODEBUGP(ODEB_CALL,( KERN_INFO "PING: pthread_make_periodic_np\n"));
	pthread_make_periodic_np( pthread_self(), gethrtime(), period );

	do {
		ODEBUGP(ODEB_CALL,("PING: wait next period\n"));
		pthread_wait_np();
		if ( my_stream_p != NULL ) {
			if ( (mbp = ocs_get_mbuf(OCS_WAIT)) == NULL ){
				ODEBUGP(ODEB_CALL,("PING: ocs_get_mbuf error\n"));
				return(0);
			}
			ocs_put_q(&my_stream_p->s_queue_up,mbp);
			if ( ocs_get_q(&my_stream_p->s_queue_down,OCS_WAIT) == NULL ){
				ODEBUGP(ODEB_CALL,("PING: ocs_get_q error\n"));
				return(0);
			}
			ocs_free_mbuf(mbp);
		}
	} while(1);
	return(0);
}


int init_module( void )
{
	int thread_status;
	pthread_attr_t attr;
	struct sched_param sched_param;

	pthread_attr_init(&attr);
	sched_param.sched_priority = 4;
	pthread_attr_setschedparam(&attr,&sched_param);
	thread_status=pthread_create(&ping_thread, &attr,ping_rt_thread,(void *)mon_message); 

if ( thread_status < 0 ){
	printk( KERN_INFO "ocera streams ping: pthread_create error.\n");
	return(-1);
	}

printk( KERN_INFO "ocera ping (version "VERSION ") sucessfully loaded.\n"
        KERN_INFO "ocera ping: Copyright (C) 2003 Pierre Morel.\n" );
return(0);
}


void cleanup_module( void )
{
pthread_delete_np (ping_thread);
printk( KERN_INFO "ocera streams ping unloaded.\n" );
return;
}

