#include <linux/config.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/socket.h>
#include <linux/in.h>

#include <pthread.h>
#include <time.h>
#include <unistd.h>

#include <asm/system.h>
#include <asm/io.h>

#include <rtl.h>
#include <rtl_fifo.h>

#define RTL_SPIN_LOCK SPIN_LOCK_UNLOCKED
#include <rtl_sync.h>
#include "../ocs.h"


MODULE_AUTHOR("Pierre Morel <pmorel@mnis.fr>");
MODULE_DESCRIPTION("Ocera Network pong module example");
MODULE_LICENSE("GPL");

/*
 * Initialize the module, get queues and wait for data
 */

int period = OCS_PERIOD*100 ;
int count = 0 ;

pthread_t	pong_thread;
char * mon_message = "salut";
ocs_t	* my_stream_p;

static int pid;
static struct semaphore S_sem;

static int *pong_lin_thread(void *t){
	ocs_t			*sp;
	ocera_msgbuf_t *mbp;
	int				sig;

	ODEBUGP(ODEB_CALL,( KERN_INFO "LIN_PONG: start %s\n",t));

	ODEBUGP(ODEB_CALL,( KERN_INFO "LIN_PONG: pthread_make_periodic_np\n"));
	daemonize();

	sprintf(current->comm,"LIN_PONG");
	if ( (sp=ocs_register("pong",ST_LINUX_TYPE)) == NULL) {
		ODEBUGP(ODEB_CALL,( KERN_INFO "LIN_PONG: ocs_register error\n"));
		return(0);
	}
	my_stream_p = sp ;
	do {
		ODEBUGP(ODEB_CALL,("LIN_PONG: loop sp = %p\n",sp));
		if( ( mbp = ocs_get_q(&sp->s_queue_up, OCS_WAIT)) == NULL) {
			ODEBUGP(ODEB_CALL,("LIN_PONG: ocs_get_q error\n"));
			return(0);
		}
		if ( (sig=signal_pending(current)) !=0 )  {
			ODEBUGP(ODEB_CALL,("onetd - signal received %d\n",sig));
			up(&S_sem);
			return(0);
		}
		if ( count++ < 5 )
			kernel_thread(pong_lin_thread,NULL, CLONE_FS | CLONE_FILES | CLONE_SIGHAND);
		ocs_put_q(&sp->s_queue_down,mbp);
		ODEBUGP(ODEB_CALL,("LIN_PONG: wait next period\n"));
		} while(1);
	return(0);
}



int init_module( void )
{

	pid=kernel_thread(pong_lin_thread,NULL, CLONE_FS | CLONE_FILES | CLONE_SIGHAND);

if (pid < 0 ){
	printk( KERN_INFO "ocera streams pong: pthread_create error.\n");
	return(-1);
	}

printk( KERN_INFO "ocera pong (version "VERSION ") sucessfully loaded.\n"
        KERN_INFO "ocera pong: Copyright (C) 2003 Pierre Morel.\n" );
return(0);
}


void cleanup_module( void )
{
	kill_proc(pid,SIGKILL,1);
	down(&S_sem);
	kill_proc(2, SIGCHLD, 1);
	printk( KERN_INFO "ocera streams pong unloaded.\n" );
	return;
}

