/*
 * OCERA STREAMS
 *
 *
 */
#include <linux/config.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/proc_fs.h>

#include <pthread.h>
#include <time.h>
#include <unistd.h>

#include <asm/system.h>
#include <asm/io.h>
#include <asm/uaccess.h>


#define RTL_SPIN_LOCK SPIN_LOCK_UNLOCKED
#include <rtl.h>
#include <rtl_printf.h>
#include <rtl_sync.h>
#include <semaphore.h>
#include "ocs.h"

struct proc_dir_entry *proc_ocera;
struct proc_dir_entry *proc_ocs;
struct proc_dir_entry *proc_ocs_slot;

static int proc_calc_metrics(char *page, char **start, off_t off,
                 int count, int *eof, int len)
{
    if (len <= off+count) *eof = 1;
    *start = page + off;
    len -= off;
    if (len>count) len = count;
    if (len<0) len = 0;
    return len;
}


int ocs_read_proc_state(char *page, char **start, off_t off,
                 int count, int *eof, void *data)
{
int len;

len=sprintf(page,"%02lx %0d\n",ocs_state,ocs_free);
return proc_calc_metrics(page, start, off, count, eof, len);
}

int ocs_read_proc_msgbuf(char *page, char **start, off_t off, int count, int *eof, void *data)
{
int len;

len=sprintf(page,"%0d\n",mbp_free_q.q_count);
return proc_calc_metrics(page, start, off, count, eof, len);
}

int ocs_read_proc_slot(char *page, char **start, off_t off,
                 int count, int *eof, void *data)
{
	int len=0;
	char *n_up="none";
	char *n_down="none";
	ocs_t *osp = (ocs_t *)data;
	ocs_t *up;
	ocs_t *down;

	up = osp->s_up;
	down = osp->s_down;
	if ( up != (ocs_t *) 0)
			n_up = up->s_name;
	if ( down != (ocs_t *) 0)
			n_down = down->s_name;
	
	len=sprintf(page,"%10s %10s %10s %0lx %0lx %0d %0d %0d %0d %0d %0d\n",
			   	osp->s_name ,
			   	n_up ,
			   	n_down,
			  	osp->s_state,
			   	osp->s_type, 
				osp->s_queue_up.q_count,
				osp->s_queue_up.q_soft,
			   	osp->s_queue_down.q_count,
			   	osp->s_queue_down.q_soft,
			   	osp->s_queue_wait.q_count,
			   	osp->s_queue_wait.q_soft
	   	   );
	return proc_calc_metrics(page, start, off, count, eof, len);
}

int ocs_deblevel_read (char *page, char **start, off_t off, int count, int *eof, void *data){
	int len;
	len=sprintf(page,"%0d\n",ocs_deblevel);
	return proc_calc_metrics(page, start, off, count, eof, len);
}

int ocs_deblevel_write(struct file * filp, const char * buffer, unsigned long count, void * data){
	unsigned int level;
	if (!capable(CAP_SYS_ADMIN) ){
		return -EACCES;
	}
	copy_from_user(&level, buffer, 1);
	level -= '0';
	if ( level  <= ODEB_FULL) ocs_deblevel = level;
	return(1);
}

